# coding: utf-8
import os
import spur
import shlex
import logging
import simplejson as json
from unittest2 import TestCase
from branchesv import branches, utils

logger = logging.getLogger('deployv')


class TestBranches(TestCase):

    @classmethod
    def setUpClass(cls):
        cls.gitbranch = branches.GitBranch()
        cls.shell = spur.LocalShell()
        cls._clone_test_repos('branch_test/')
        cls.branches_config = utils.simplify_path(
            cls.gitbranch.get_branches('branch_test/')
        )

    @staticmethod
    def _clone_test_repos(path_to_clone):
        shell = spur.LocalShell()
        repos = [('https://github.com/Vauxoo/deploy-templates.git', 'deploy-templates'),
                 ('https://github.com/Vauxoo/docker_entrypoint.git', 'docker_entrypoint'),
                 ('https://github.com/Vauxoo/pylint-conf', 'docker_entrypoint/pylint')]
        for repo in repos:
            clone = 'git clone {repo} {path}'.format(
                repo=repo[0],
                path=os.path.join(path_to_clone, repo[1]))
            shell.run(shlex.split(clone))

    def test_10_is_branch(self):
        is_branch = self.gitbranch.is_branch('branch_test/deploy-templates')
        self.assertTrue(is_branch)

    def test_20_get_recursive_branches(self):
        path = 'branch_test/'
        repos_names = []
        repos = self.gitbranch.get_recursive_branches(path)
        self.assertIsInstance(repos, list)
        self.assertEquals(len(repos), 3)
        for repo in repos:
            repos_names.append(repo['name'])
        self.assertIn('deploy-templates', repos_names)
        self.assertIn('docker_entrypoint', repos_names)
        self.assertIn('pylint-conf', repos_names)

    def test_30_get_branches(self):
        recursive_repos = self.gitbranch.get_branches('branch_test/docker_entrypoint',
                                                      recursive=True)
        self.assertEquals(len(recursive_repos), 2)
        norecursive_repos = self.gitbranch.get_branches('branch_test/docker_entrypoint',
                                                        recursive=False)
        self.assertEquals(len(norecursive_repos), 1)

    def test_40_set_branch(self):
        clone_branch_latest = self.gitbranch.set_branch(self.branches_config[0],
                                                        'branch_test')
        self.assertEqual(clone_branch_latest.get('name'),
                         self.branches_config[0].get('name'))
        self.assertTrue(os.path.exists(os.path.join('branch_test',
                                                    self.branches_config[0].get('path'))))
#        cmd = ('git -C branch_test/test40/{repo}/ log'
#               ' --pretty=format:\'%H\' -n 1'.format(repo=clone_branch_latest['name']))
#        res = self.shell.run(shlex.split(cmd))
#        current_commit = res.output
#        self.assertEqual(current_commit.strip(), self.branches_config[0].get('commit'))

    def test_60_action_file(self):
        branches.action_file('branch_test', 'action_file',
                             'branch_test')
        self.assertTrue(os.path.exists('branch_test/action_file'))

    def test_99_cleanup(self):
        utils.clean_files('branch_test')



# coding: utf-8

import os
import spur
import shlex
from unittest import TestCase
from branchesv import utils, branches


class TestUtils(TestCase):

    @classmethod
    def setUpClass(cls):
        cls.gitbranch = branches.GitBranch()
        cls.shell = spur.LocalShell()
        cls.shell.run(shlex.split('git clone https://github.com/Vauxoo/deploy-templates.git'
                                  ' test_utils/deploy-templates'))
        cls.original_branches = cls.gitbranch.get_branches('test_utils')

    def test_10_load_json(self):
        res = utils.load_json('tests/files/branches.json')
        self.assertTrue(res)
        self.assertIsInstance(res, list)

    def test_15_load_json_exception(self):
        with self.assertRaises(IOError):
            utils.load_json('testing/config')

    def test_20_save_json(self):
        content = utils.load_json('tests/files/branches.json')
        res = utils.save_json(content, 'test_utils/save_test.json')
        new_json = utils.load_json('test_utils/save_test.json')
        self.assertTrue(os.path.exists(res))
        self.assertEquals(content, new_json)

    def test_25_save_json_exception(self):
        with self.assertRaises(IOError):
            utils.save_json('info', 'testing/test.json')

    def test_30_simplify_path(self):
        copy_branches = utils.copy_list_dicts(self.original_branches)
        res = utils.simplify_path(copy_branches)
        full_path = 'test_utils/{path}'.format(path=res[0].get('path'))
        self.assertNotEqual(res[0].get('path'), self.original_branches[0].get('path'))
        self.assertEquals(full_path, self.original_branches[0].get('path'))
        self.assertEquals(len(res[0].get('path').split('/')), 1)

    def test_40_name_from_url(self):
        res = utils.name_from_url('https://github.com/odoo/odoo.git')
        self.assertEqual(res, 'odoo')

    def test_50_copy_list_dicts(self):
        copy = utils.copy_list_dicts(self.original_branches)
        self.assertEquals(copy, self.original_branches)
        copy[0].update({'name': 'new_name'})
        self.assertNotEqual(copy, self.original_branches)

    def test_60_set_message(self):
        res_success = utils.set_message(self.original_branches[0])
        res_fatal = utils.set_message(self.original_branches[0], 'fatal: no such branch')
        self.assertEquals(res_success.get('exit_code'), 0)
        self.assertEquals(res_fatal.get('exit_code'), 1)

    def test_70_is_iterable(self):
        self.assertTrue(utils.is_iterable([1, 2]))
        self.assertTrue(utils.is_iterable((1, 2)))
        self.assertFalse(utils.is_iterable('1, 2'))
        self.assertFalse(utils.is_iterable(b'1, 2'))

    def test_90_clean_files(self):
        files = ['/', 'test_utils/save_test.json', 'test_utils/']
        utils.clean_files(files)
        self.assertFalse(os.path.exists('test_utils/save_test.json'))
        self.assertFalse(os.path.exists('test_utils/'))
        self.assertTrue(os.path.exists('/'))



#!/usr/bin/python
# coding: utf-8
""" This script loads branches information to a Json file
and reconstruct branches from such files.

The file format is as follows::

    [
        {
            "branch":"branch_name",
            "commit":"commit_hash",
            "depth":1,
            "is_dirty":false,
            "name":"repo_name",
            "path":"path/to/clone",
            "repo_url":{
                "origin":"git@github.com:Vauxoo/repo_name.git"
            },
            "type":"git"
        },
        {
            "branch":"branch_name",
            "commit":"commit_hash",
            "depth":1,
            "is_dirty":false,
            "name":"anotherrepo_name",
            "path":"path/to/clone/other/repo",
            "repo_url":{
                "origin":"git@github.com:Vauxoo/anotherrepo_name.git"
            },
            "type":"git"
        }
    ]

Is a list of dictionaries containing all the information of the repos where:

*branch*: Name of the branch
*commit*: commit hash you wish in the build (leave blank for the latest)
*depth*: --depth option (see git doc)
*is_dirty*: True if there is any local change that have not been commited yet
*name*: repo name
*path*: path where the repo will be cloned
*repo_url*: Github url where the repo is stored o where it was gotten from
"""

import chardet
import os
import logging
import git
from git import Repo
from . import utils

logging.basicConfig(
    level=logging.DEBUG,
    format='%(asctime)s - %(levelname)-5s - %(name)s.%(module)s.%(funcName)s - %(message)s'
)
_logger = logging.getLogger('Branchesv')


class GitBranch(object):
    """ This class provides manipulation of git repositories using
    lists of config dictionaries. Each dictionary contains the following
    information of a git branch:

    *path*: Path of branch location, usually relative to a common origin
    *name*: Name of repository
    *branch*: Branch to be used
    *commit*: Hash of the commit pointed by HEAD
    *is_dirty*: Status of the branch
    *depth*: Depth of the cloned branch. False by default.
    *type*: Always 'git'.
    *repo_url*: Urls of remote origins of the repository
    """

    def __init__(self, key_path=False):
        self.identity_file = '-i %s' % (key_path) if key_path else ''
        self.ssh_command = ('ssh %s -o UserKnownHostsFile=/dev/null '
                            ' -o StrictHostKeyChecking=no '
                            ' -o BatchMode=yes') % (self.identity_file)

    def __get_branch(self, path):
        """ Gets required information of a repository

            :param path: Path of .git directory
            :return: Config dictionary
        """
        info = {}
        repo = Repo(path)
        info.update({'path': str(os.path.dirname(path)),
                     'branch': str(repo.head.ref),
                     'commit': str(repo.head.reference.commit),
                     'is_dirty': repo.is_dirty(),
                     'name': utils.name_from_url(str(repo.remotes[0].url)),
                     'depth': 1})
        urls = {}
        remotes = repo.remotes
        for remote in remotes:
            urls.update({remote.name: remote.url})
        info.update({'repo_url': urls})
        return info

    def is_branch(self, path):
        """ Checks if a given path exists and is a git branch or not

            :param path: Path to be checked
            :return: True if path exists and is a git branch. False otherwise.
        """
        isbranch = False
        if os.path.exists(path):
            if os.path.isdir(path) and not os.path.islink(path):
                if ".git" in os.listdir(path):
                    isbranch = True
        return isbranch

    def get_recursive_branches(self, path):
        """ Gets information of all existing repositories in a
        directory in a recursive way, searching all the inner
        paths

            :param path: Common path that contains the repositories
            :param verbose: A param that allows to write in the logger or not
            :return: List of config dictionaries
        """
        res = []
        if os.path.exists(path) and os.path.isdir(path) and \
           not os.path.islink(path):
            for element in os.listdir(path):
                try:
                    element_path = os.path.join(path, element)
                except UnicodeDecodeError:
                    encoding = chardet.detect(element)['encoding']
                    element_path = os.path.join(path, element.decode(encoding))
                if os.path.isdir(element_path) and not os.path.islink(element_path):
                    if element == '.git':
                        res.append(self.__get_branch(element_path))
                        _logger.debug("Branch collected from %s", element_path)
                    else:
                        temp_res = self.get_branches(element_path)
                        if temp_res:
                            res = res + temp_res
        return res

    def get_branches(self, path, recursive=True):
        """ Gets information of all existing repositories in a
        directory

            :param path: Common path that contains the repositories
            :param verbose: A param that allows to write in the logger or not
            :param recursive: Sets the search as recursive in the path
            :return: List of config dictionaries
        """
        if recursive:
            res = self.get_recursive_branches(path)
        else:
            res = []
            if self.is_branch(path):
                git_path = os.path.join(path, ".git")
                res.append(self.__get_branch(git_path))
        return res

    def __clone(self, path, branch):
        """ Clones a repository in the specified path

            :param path: Path where will be cloned the repository
            :param branch: Config dictionary of the repository
            :return: Message dictionary
        """
        depth = branch.get('depth', False)
        clone_path = os.path.join(path, branch['path'])
        url = branch['repo_url'].get('origin', list(branch['repo_url'].values())[0])
        _logger.debug('Cloning repo %s - branch %s', branch['name'], branch['branch'])
        kwargs = {'env': {'GIT_SSH_COMMAND': self.ssh_command}}
        if branch.get('branch', False):
            kwargs.update({'depth': depth, 'branch': branch['branch']})
        try:
            repo = Repo.clone_from(url, clone_path, **kwargs)
            branch.update({'branch': str(repo.head.ref),
                           'curr_commit': str(repo.head.reference.commit)})
            _logger.debug('Repo cloned %s - branch %s', branch['name'], branch['branch'])
            res = utils.set_message(branch)
        except git.exc.GitCommandError as error:
            _logger.error(error)
            res = utils.set_message(branch, error.stderr)
        return res

    def set_branch(self, branch, path):
        """ Clones a single repository in the specified path with the
        specified configuration

            :param branch: Config dict of the repository
            :param path: Path where will be cloned the repository
            :return: Message dictionary
        """
        _logger.info('Setting repo %s - branch %s', branch['name'], branch['branch'])
        if not branch.get('path', False):
            branch.update({'path': branch['name']})
        clone_path = os.path.join(path, branch['path'])
        if os.path.exists(clone_path) and self.is_branch(clone_path):
            if branch.get('commit', False):
                res = self.reset(branch, path)
            else:
                res = self.__pull(branch, clone_path)
            if res['exit_code'] != 0:
                branch.update({'depth': False})
                utils.clean_files(clone_path)
                res = self.__clone(path, branch)
                if branch.get('commit', False):
                    res = self.reset(branch, path)
        elif branch.get('commit', False):
            branch.update({'depth': False})
            res = self.__clone(path, branch)
            if res['exit_code']:
                return res
            res = self.reset(branch, path)
        else:
            res = self.__clone(path, branch)
        if branch.get('curr_commit', False):
            _logger.info('Repo set %s - branch %s - sha %s', branch['name'], branch['branch'],
                         branch['curr_commit'])
            branch.pop('curr_commit')
        return res

    def reset(self, branch, path):
        _logger.debug('Resetting repo %s to commit %s', branch['path'], branch['commit'])
        reset_path = os.path.join(path, branch['path'])
        repo = Repo(reset_path)
        try:
            repo.git.reset(branch['commit'], "--hard")
            branch.update({'curr_commit': str(repo.head.reference.commit)})
            _logger.debug('Repo %s reset to commit %s', branch['path'], branch['commit'])
            res = utils.set_message(branch)
        except git.exc.GitCommandError as error:
            _logger.error(error)
            res = utils.set_message(branch, error.stderr)
        return res

    def __pull(self, branch, path=False):
        """ Pulls in the repo the new information from the origin url
            :param path: Path of the repository to be pulled
            :param branch: Config dict of the repository
            :return: Message dictionary
        """
        if not path:
            path = branch['path']
        url = branch['repo_url'].get('origin', list(branch['repo_url'].values())[0])
        try:
            repo = Repo(path)
            repo.git.update_environment(GIT_SSH_COMMAND=self.ssh_command)
            if not branch.get('branch', False):
                branch.update({'branch': str(repo.head.ref)})
            for remote in repo.remotes:
                if remote.url == url:
                    repo_index = repo.remotes.index(remote)
                    repo.remotes[repo_index].pull(branch['branch'])
                    branch.update({'curr_commit': str(repo.head.reference.commit)})
                    res = utils.set_message(branch)
                    break
            else:
                error_message = (
                    'The remote {remote} specified for the repository {repo} does not'
                    ' match any of the remotes of the repo that is already cloned in {path}.'
                    ' The repository {repo} will be cloned again'.format(remote=url,
                                                                         repo=branch['name'],
                                                                         path=path)
                    )
                _logger.warning(error_message)
                res = utils.set_message(branch, 'fatal: wrong remote')
        except git.exc.GitCommandError as error:
            res = utils.set_message(branch, error.command[0])
        except git.exc.NoSuchPathError:
            _logger.error("fatal: No Such Path %s", branch['path'])
            res = utils.set_message(branch, "fatal: No Such Path {path}".format(path=path))
        return res

    def pull(self, branch):
        """ Updates a single branch to a specified commit

            :param path: Common path of the repositories
            :param branch: Branches to be updated
            :return: Message dictionary
        """
        _logger.debug('Pulling latest changes for %s', branch['name'])
        res = self.__pull(branch)
        _logger.debug('Latest changes pulled for %s', branch['name'])
        return res


def action_file(tmp_dir, action, path):
    """ This function saves branches configuration in a json file, this is
    intended to be used before or after some action is executed on the
    instance

        :param dir: Directory where file will be dumped
        :param action: Reason for dumping the file
        :param path_list: List of paths where the branches are located
        :param branches: List of config dictionaries to be dumped
    """
    _logger.debug('Creating %s', action)
    b_info = []
    branches = GitBranch()
    temp_res = branches.get_branches(path)
    b_info = b_info + temp_res
    b_info = utils.simplify_path(b_info)
    file_action = os.path.join(tmp_dir, action)
    utils.save_json(b_info, file_action)


def save(json_file, path, recursive):
    """save command
    """
    if path == '.':
        path = os.getcwd()
    path_list = path.split(',')
    gitbranch = GitBranch()
    for paths in path_list:
        b_info = gitbranch.get_branches(paths, recursive=recursive)
        b_info_simple = utils.simplify_path(b_info)
        utils.save_json(b_info_simple, json_file)


def load(repos, path, tmp, key_path=False):
    if path == '.':
        path = os.getcwd()
    path_list = path.split(',')
    gitbranch = GitBranch(key_path)
    for paths in path_list:
        action_file(tmp, "pre_process.json", paths)
        for repo in repos:
            res = gitbranch.set_branch(repo, paths)
            if res.get('exit_code') > 0:
                return res
        action_file(tmp, "post_process.json", paths)


def pull(path, tmp, recursive, key_path=False):
    if path == '.':
        path = os.getcwd()
    path_list = path.split(',')
    gitbranch = GitBranch(key_path)
    for paths in path_list:
        b_info = gitbranch.get_branches(path, recursive=recursive)
        action_file(tmp, "pre_process.json", paths)
        for branch in b_info:
            gitbranch.pull(branch)
        action_file(tmp, "post_process.json", paths)



import os
import locale
import shutil
import simplejson as json
import logging
import re
from six import string_types, binary_type


logging.basicConfig(
    level=logging.DEBUG,
    format='%(asctime)s - %(levelname)-5s - %(name)s.%(module)s.%(funcName)s - %(message)s'
)
_logger = logging.getLogger('Branchesv')


def save_json(info, filename):
    """ Save info into Json file.
        :param info: Object to be saved
        :param filename: Name of Json file
        :return: Absolute path of Json file
    """
    if not os.path.isabs(filename):
        filename = os.path.abspath(filename)
    try:
        with open(filename, 'w') as fout:
            json.dump(info, fout, sort_keys=True, indent=4, ensure_ascii=False,
                      separators=(',', ':'))
    except IOError as error:
        _logger.error(error)
        raise
    return filename


def load_json(filename):
    """ Load info from Json file
        :param filename: Name of Json file
        :return: Object loaded if successful, None otherwise
    """
    try:
        with open(filename, "r") as dest:
            info = json.load(dest)
    except IOError as error:
        _logger.error(error)
        raise
    return info


def is_iterable(obj):
    """ Method that verifies if an object is iterable and not a string

    :param obj: Any object that will be tested if is iterable
    :return: True or False if the object can be iterated
    """
    return hasattr(obj, '__iter__') and not isinstance(obj, (str, bytes))


def clean_files(files):
    """ Remove unnecessary and temporary files
       :param files: A list of absolute or relative paths thar will be erased
    """
    items = files if is_iterable(files) else [files]
    for item in items:
        if item != "/":
            if os.path.isfile(item):
                os.remove(item)
            elif os.path.isdir(item):
                shutil.rmtree(item)
        else:
            _logger.error("Invalid target path: '/'")


def simplify_path(branches_info):
    """ This function deletes all common directories in branches' path
        :param branches_info: List of dictionaries with branches' info
        :return: List of dictionaries with branches' info
    """
    if len(branches_info) > 1:
        repeated = True
        while repeated:
            piece_path = []
            for branch in branches_info:
                piece_path.append(branch['path'].split('/', 1)[0])
            word = piece_path[0]
            repeated = True
            for each in piece_path:
                if each != word:
                    repeated = False
            if repeated:
                for branch in branches_info:
                    branch.update({'path': branch['path'].split('/', 1)[1]})
    elif branches_info:
        branch = branches_info[0]
        branch.update({'path': os.path.basename(branch['path'])})
    return branches_info


def name_from_url(url):
    """ Takes name of a GIT repo from origin url
        :param url: Url of private GIT repo
        :returns: Repo's name
    """
    name = url.split('/')[-1].split('.')[0]
    return name


def copy_list_dicts(lines):
    res = []
    for line in lines:
        dict_t = {}
        for keys in line.keys():
            dict_t.update({keys: line[keys]})
        res.append(dict_t.copy())
    return res


def set_message(branch, str_msg=None):
    """ Sets the exit message for GitBranch actions like clone, pull
    and update
        :param branch: The repo that's being used
        :param str_msg: Posible string message received from an action
        :return: Message dictionary containing:
            - name: Name of the repo
            - msg: Message to be returned
            - exit_code: Code or return
    """
    url = branch['repo_url'].get('origin', list(branch['repo_url'].values())[0])
    fatal_error = None
    if str_msg:
        fatal_regexp = re.compile(r'fatal: (.*)')
        fatal_error = fatal_regexp.search(decode(str_msg))
    if not fatal_error:
        res = {"name": branch['name'],
               "exit_code": 0,
               "msg": "Successful action"}
    else:
        res = {
               "name": branch['name'],
               "exit_code": 1,
               "msg": "{error} while cloning {repo} from {url}".format(error=fatal_error.group(0),
                                                                       repo=branch['name'],
                                                                       url=url)}
    return res


def decode(string, errors='replace'):
    """Decodes the `string` parameter if it's a byte string or returns it if
    it's a string. This is used to convert all the strings returned by the
    git library to a standard way.
    """
    try:
        locale.setlocale(locale.LC_ALL, '')
        charset = locale.getlocale(locale.LC_CTYPE)[1]
    except locale.Error:
        charset = 'ascii'
    if isinstance(string, binary_type) and not isinstance(string, string_types):
        return string.decode(encoding=charset, errors=errors)
    elif hasattr(string, 'decode'):
        return string.decode(charset)
    return string


def re_match_git_url(url):
    """Parse the repository's url to get it's parts such as the protocol, host,
    token, organization and the repository's name, if it not match return a empty dict.
    The url can be in either format:
        https://github.com/organization/repo
        git@github.com:organization/repo.git
        https://token:token@git.vauxoo.com/organization/repo

    :param url: The git url where get ehe information.
    :type: str
    :return: Return a dict with the information got of the url.
    :rtype: dict
    """
    regex = (r'(git@|http[s]?:\/\/)((?P<token>.*)@)?(?P<host>.*)'
             r'([:\/]{1})(?P<organization>.*)/(?P<repo>.*)(\.git)')
    res = re.match(regex, url)
    return res and res.groupdict() or {}



# -*- coding: utf-8 -*-

import click
from . import branches, utils


@click.group()
def cli():
    pass


@cli.command()
@click.option("-f", "--json-file", help="Json file to use", required=True)
@click.option("-p", "--path", required=True,
              help="Path of GIT Repo, actual dir by default. May be a list for save option",
              default=None)
@click.option("--recursive/--no-recursive", default=True,
              help="Sets the search of repositories as recursive or no recursive. Recursive by default")
def save(json_file, path, recursive):
    """save command
    """
    branches.save(json_file, path, recursive)


@cli.command()
@click.option("-f", "--json-file", help="Json file to use", required=True)
@click.option("-p", "--path", required=True,
              help="Path of GIT Repo, actual dir by default. May be a list for save option",
              default=None)
@click.option("--tmp", help="Temporary directory for branches files",
              default="/tmp")
@click.option("-k", "--key-ssh", help="The SSH private key that will use to clone/pull the repositories",
              default=False)
def load(json_file, path, tmp, key_ssh):
    repos = utils.load_json(json_file)
    branches.load(repos, path, tmp, key_ssh)


@cli.command()
@click.option("-p", "--path",
              help="Path of GIT Repo, actual dir by default. May be a list",
              default=None, required=True)
@click.option("--tmp", help="Temporary directory for branches files",
              default="/tmp")
@click.option("--recursive/--no-recursive", default=True,
              help="Sets the search of repositories as recursive or no recursive. Recursive by default")
@click.option("-k", "--key-ssh", help="The SSH private key that will use to pull the repository",
              default=False)
def pull(path, tmp, recursive, key_ssh):
    branches.pull(path, tmp, recursive, key_ssh)



#!/usr/bin/env python
# -*- coding: utf-8 -*-


from setuptools import setup, find_packages


with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

requirements = open('requirements.txt').readlines()

test_requirements = open('requirements_dev.txt').readlines()

setup(
    name='branchesv',
    version='0.2.9',
    description="Python package to get the information of all the repos in a directory",
    long_description=readme + '\n\n' + history,
    author="Vauxoo",
    author_email='joseangel@vauxoo.com',
    url='https://git.vauxoo.com/vauxoo/branchesv',
    packages=find_packages(),
    package_dir={'branchesv': 'branchesv'},
    include_package_data=True,
    install_requires=requirements,
    license="ISCL",
    zip_safe=False,
    keywords='branchesv',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: ISC License (ISCL)',
        'Natural Language :: English',
        "Programming Language :: Python :: 2",
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',

    ],
    test_suite='tests',
    tests_require=test_requirements,
    entry_points='''
        [console_scripts]
        branchesv=branchesv.branchesvcmd:cli
    ''',
)
